numLanes = 5
halfLanes = (numLanes -1) / 2
minimumBlockSpace=6

minPos = -6.0
maxPos = 6.0
range = maxPos - minPos
graceRange = 1.0 -- allow about a third of a lane of slack to get full points

GameplaySettings{
		usepuzzlegrid = false,
        greypercent=0,
        colorcount=2,
        usetraffic = true,
        automatic_traffic_collisions = false, -- the game shouldn't check for block collisions since we'll be doing that ourselves in this script
        jumpmode="none",
        matchcollectionseconds=1.5,
        greyaction="normal", -- "eraseall"  -- "eraseblock"

		--track generation settings
		gravity=-.45,
        playerminspeed = 0.1,--so the player is always moving somewhat
        playermaxspeed = 2.9,--2.5
        minimumbestjumptime = 2.5,--massage the track until a jump of at least this duration is possible
        uphilltiltscaler = 1,--set to 1 for normal track. higher for steeper
        downhilltiltscaler = 1,--set to 1 for normal track. higher for steeper
        uphilltiltsmoother = 0.03,
        downhilltiltsmoother = 0.06,
        useadvancedsteepalgorithm = false,--set false for a less extreme track
        alldownhill = false,
		--end track generation settings
		
        --trafficcompression=.5,
}

SetSkinProperties{
	lanedividers={-4.5,-1.5,1.5,4.5},
	shoulderlines={minPos,maxPos},
	trackwidth = 11.5
}

player={
	score=0,
	prevInput={},
	iPrevRing=0,
	hasFinishedScoringPrevRing=false,
	uniqueName = "Ship",
	num=1,
	prevFirstBlockCollisionTested = 1,
	pos = {0,0,1.5},
	posCosmetic = {0,0,1.5},
	controller = "mouse",
	points = 0, -- used for accumulating points this player earns at each match collection. temp var
	lane = 0
}

function OnTrackCreated(theTrack)--track is created before the traffic
	track = theTrack --store a global copy of the track to maybe use later
	-- when you return a track table from this function the game will read and apply any changes you made
end

lanespace = 3
half_lanespace = 1.5

blocks = blocks or {}
blockNodes = blockNodes or {}

function OnTrafficCreated(theTraffic)
	half_lanespace = lanespace / 2

	traffic = theTraffic

	local newTraffic = {}
	local threshold = 0.015
	
    for i = 1, #traffic do
		if traffic[i].strength > threshold then
			newTraffic[#newTraffic+1] = traffic[i]
		end
    end
	
	for i = 1, #newTraffic do
		local lane = newTraffic[i].lane
    	local strafe = lane*lanespace
    	local offset = {strafe,0,0}
		local block = {}
		newTraffic[i].type = i % 2
		block.lane = lane
		block.strafe = strafe
		block.hidden = false
		block.tested = false
		block.type = newTraffic[i].type
		blocks[#blocks+1]=block
		blockNodes[#blockNodes+1] = newTraffic[i].impactnode
		if i < #newTraffic then
			if newTraffic[i+1].impactnode < newTraffic[i].impactnode + minimumBlockSpace then
				newTraffic[i+1].impactnode = newTraffic[i].impactnode + minimumBlockSpace
			end
		end
	end
	
    return newTraffic
end

function OnSkinLoaded()-- called after OnTrafficCreated. The skin script has loaded content.
	CreateClone{name=player.uniqueName, prefabName="Vehicle", attachToTrackWithNodeOffset=-1, transform={pos=player.pos}}

	SetPuzzle{trackoffset=-.25}

	HideBuiltinPlayerObjects() -- hide the game-controlled vehicle since we're using script-controlled vehicles instead. Also hides the game-controlled surfer

	SetCamera{ -- calling this function (even just once) overrides the camera settings from the skin script
		nearcam={
			pos={0,4,-3.50475},
			rot={38,0,0},
			strafiness = 0
		},
		farcam={
			pos={0,12.8,-3.50475},
			rot={41,0,0},
			strafiness = 0
		}
	}
end

score = 0 --the global score shared by all players co-operatively


function GetChainMultiplier( chainCount )
	if chainCount >= 100 then
		return 4
	elseif chainCount < 2 then
		return 1
	end
	
	return 3.2 / - ( ( 0.21 * ( chainCount - 1 ) ) + 1 ) + 4.14545
end

chainMultiplier = 1.0
iCurrentRing = 0 --Update function keeps this current
blocksToHide = {}
currentStreak = 0
maxStreak = 0
function Collide(player)
	local strafe = player.pos[1]
	local playerLane = 0;

	local absStrafe = math.abs(strafe)

	for i=2,0,-1 do
		if absStrafe > ((lanespace * i) + half_lanespace) then
			playerLane = i+1
			break
		end
	end

	if strafe < 0 then
		playerLane = -playerLane
	end

	local maxRing = iCurrentRing + 2
	local foundFirst = false
	for i=player.prevFirstBlockCollisionTested,#blockNodes do
		if not blocks[i].tested then
			if blockNodes[i] <= maxRing then
				if not foundFirst then
					player.prevFirstBlockCollisionTested = i
					foundFirst = true
				end

				if blockNodes[i] <= math.floor(iCurrentRing) then
					local multiplier = 0
					local absShipPos = strafe - minPos
					local absBlockPos = blocks[i].strafe - minPos
					
					if blocks[i].type == 0 then
					
						if blocks[i].strafe > strafe then
							currentStreak = currentStreak + 1
							if currentStreak > maxStreak then
								maxStreak = currentStreak
							end
							multiplier = 1 - ( (absBlockPos - absShipPos - graceRange) / range )
							if multiplier > 1 then
								multiplier = 1
							end
						else
							currentStreak = 0
						end
						
					elseif blocks[i].type == 1 then
					
						if blocks[i].strafe < strafe then
							currentStreak = currentStreak + 1
							if currentStreak > maxStreak then
								maxStreak = currentStreak
							end
							
							multiplier = 1 - ( (absShipPos - absBlockPos - graceRange) / range )
							if multiplier > 1 then
								multiplier = 1
							end
						else
							currentStreak = 0
						end
						
					end
					
					blocks[i].tested = true
					if multiplier > 0 then
						chainMultiplier = GetChainMultiplier( currentStreak )
						
						score = score + (1000 * multiplier * chainMultiplier)
						SetGlobalScore{score=score,showdelta=true}
					end
					
				end
			else
				break --stop the loop once we get to a block way past the player
			end
		end
	end
	SetScoreboardNote{text="Current streak: " .. currentStreak .. "  Longest streak: " .. maxStreak }
end

mouseSpeed = .42
cosmeticStrafeSpeed = 15
maxStrafe = halfLanes * 3

function UpdatePlayer(player, input, dt, keys)
	local mouseInput = input["mouse"]
	local mouseHorizontal = mouseInput["x"]
	local keyHorizontal = input["Horizontal"]
	local prevKeyHorizontal = player.prevInput["Horizontal"]

	if(player.controller=="mouse") then
		if keyHorizontal ~= 0 then
			player.controller = "key"
		end
	elseif mouseHorizontal~=0 then
		if player.num==1 then -- only player1 has the option of mouse control
			player.controller = "mouse"
		end
	end

	if player.controller=="mouse" then
		if dt>0 then --don't move when the game is paused
			player.pos[1] = math.min(maxStrafe, math.max(-maxStrafe, player.pos[1] + mouseHorizontal * mouseSpeed))
		end

		player.posCosmetic[1] = player.pos[1]
	else --key mode (keyboard or gamepad)

		if keyHorizontal > 0.5 and prevKeyHorizontal <= 0.5 then 
			player.lane = player.lane + 1
		elseif keyHorizontal < -0.5 and prevKeyHorizontal >= -0.5 then
			player.lane = player.lane - 1 
		end

		if player.lane > halfLanes then
			player.lane = halfLanes
		elseif player.lane < -halfLanes then
			player.lane = -halfLanes
		end

		player.pos[1] = lanespace * player.lane

		player.posCosmetic[1] = player.posCosmetic[1] + cosmeticStrafeSpeed * dt * (player.pos[1] - player.posCosmetic[1])
	end

	SendCommand{command="SetTransform", name=player.uniqueName, param={pos=player.posCosmetic}}

	Collide(player)

	player.prevInput = input
end

function Update(dt, tracklocation, playerstrafe, input) --called every frame
	iCurrentRing = math.floor(tracklocation)
	local playersInput = input["players"]
	local player1_input = playersInput[1]

	UpdatePlayer(player, playersInput[1], dt, input["keyboard"]) -- just move the player
end

function OnRequestFinalScoring()
	
	local streakBonus = maxStreak * 1000
	
	return {
		rawscore = score,
		bonuses = { "Longest Streak Bonus: " .. streakBonus },
		finalscore = score + streakBonus
	}
end